

# In surveys conducted by statistical offices one of the main problem is to 
# have reliable estimates for domains for which the sample size is too small 
# or even equal to zero. It is the consequence of the fact that many institutions 
# need more detailed information not only for the whole country but also for some 
# specific subdomains such as geographic areas or other cross-sections. 

#  In such situations direct estimates based only on specific domain sample data 
# were insufficient because of high variability and small precision. 
# The remedy could be the methodology of small area estimation (SAE ) which plays 
# an important role in the field of modern 
# information provision, which aims to cut survey costs while lowering the 
# respondent burden.

# Thanks to their properties, SAE methods enable reliable estimation at lower 
# levels of spatial aggregation and with more specific domains, 
# where direct estimation techniques display too much estimator variance. 
# Another advantage over direct estimators is that small area estimation can 
# be used to handle cases with few or no observations for a given domain in the sample.
# Therefore it is necessary in many situations to use indirect estimates 
# that borrow strength by taking into account values of the variables of 
# interest from related areas and from that point of view increasing the 
# "effective" sample size.

# POST STRATIFIED

# Post-stratified direct estimator is a calibration (GREG) estimator when
# the auxiliary variables are post-stratum indicators.


# An unbiased estimator is obtained from a sample survey for a large
# area; when this estimate is used to derive estimates for subareas
# under the assumption that the small areas have the same
# characteristics as the large area, we identify these estimates as
# synthetic estimates.

# Synthetic assumption: small areas have same characteristic as the
# large area

# The post-stratified synthetic estimator is a basic indirect estimator.
# It assumes that data are distributed into K groups (called post-strata) 
# that cut across the domains, and such
# that the within group mean is constant across domains. 



library(sae)  

# Load data set
data("incomedata")

# Load province sizes by education levels
data(sizeprovedu)

# just Spanish
tmp_data=incomedata[incomedata$nat1==1,]

?pssynt

# Calculates post-stratified synthetic estimators of domain means using 
# the categories of a qualitative variable as post-strata  (XB)

# pssynt(y, sweight, ps, domsizebyps, data)
# Cases with NA values in y, sweight or ps are ignored

# In pssynt(), we must specify our selected post-stratifying variable in argument ps. 
# The population sizes of each crossing between domain and post-strata
# must be specified in the data frame domsizebyps, whose first column must be
# again the area codes.

colnames(sizeprovedu) <- c("provlab", "prov", "0", "1", "2", "3")
psSynt <- pssynt(y=income, sweight=weight, ps=educ,
                  domsizebyps=sizeprovedu[,-1], data=tmp_data)
head(psSynt)



#___________________________________
# Synthetic estimators with continuous covariates  \hat{Y}=XB
#___________________________________

data(cornsoybeanmeans)
data(cornsoybean)
attach(cornsoybean)
attach(cornsoybeanmeans)

?cornsoybean
?cornsoybeanmeans

Xmean <- data.frame(CountyIndex, MeanCornPixPerSeg, MeanSoyBeansPixPerSeg)
Popn  <- data.frame(CountyIndex, PopnSegments)

mod.synth<-lm(CornHec ~ CornPix + SoyBeansPix,data=cornsoybean)
synth.est<-as.matrix(cbind(1,Xmean[,2:3]))%*%as.matrix(mod.synth$coef,3,1)
synth_est=as.vector(synth.est)

data.var=as.data.frame(cbind(cornsoybean$County,predict(mod.synth)))
names(data.var)=c("County","Est")
data.var$County=factor(data.var$County)
var_est=tapply(data.var$Est, data.var$County, sd)

for(i in 1:length(var_est))
{if(is.na(var_est[i])){var_est[i]=0}}

CVs=var_est/synth_est



#??? To be continued....
# install.packages(nlme)
library(nlme)

